﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.RichTextBoxCommands;
using System.IO;
using Microsoft.Win32;
using Telerik.Windows.Documents.FormatProviders;
using Telerik.Windows.Documents.Model;
using Telerik.Windows.Controls;
using System.Windows;

namespace TelerikEditorQ1SP1
{
    public class CustomSaveCommand : RichTextBoxCommandBase
    {

        public CustomSaveCommand(RadRichTextBox editor)
            : base(editor)
        {
        }

        protected override void ExecuteOverride(object parameter)
        {
            string extension = null;
            Stream stream = null;

            if (this.ShowSaveDialog(parameter as string, out extension, out stream))
            {
                this.Save(this.AssociatedRichTextBox.Document, stream, extension);
            }
        }

        internal bool ShowSaveDialog(string exportFormat, out string extension, out Stream output)
        {
            SaveFileDialog saveDialog = new SaveFileDialog();

            if (exportFormat != null && exportFormat.Contains("|"))
            {
                saveDialog.Filter = exportFormat;
            }
            else
            {
                saveDialog.Filter = this.GetFilter(DocumentFormatProvidersManager.FormatProviders, false, false, exportFormat);
            }
            saveDialog.InitialDirectory = Path.GetFullPath(@"C:\Work\ProjectsWPF");

            bool? dialogResult = saveDialog.ShowDialog();
            if (dialogResult == true)
            {
                extension = System.IO.Path.GetExtension(saveDialog.SafeFileName);
                output = saveDialog.OpenFile();
                return true;
            }
            else
            {
                extension = null;
                output = null;
                return false;
            }
        }

        internal string GetFilter(IEnumerable<IDocumentFormatProvider> formatProviders, bool isImport, bool addAllFiles, string filter)
        {
            formatProviders = formatProviders.OrderBy(fp => fp.SupportedExtensions.FirstOrDefault());

            if (!string.IsNullOrEmpty(filter))
            {
                string[] extensions = filter.Split(',', ';').Select(e => e.Trim('.').ToLower()).ToArray();
                formatProviders = formatProviders.Where(fp => fp.SupportedExtensions.Any(ext => extensions.Contains(ext.Trim('.').ToLower())));
            }

            if (isImport)
            {
                formatProviders = formatProviders.Where(fp => fp.CanImport);
            }
            else
            {
                formatProviders = formatProviders.Where(fp => fp.CanExport);
            }

            return
                string.Join("|", formatProviders.Select(fp => GetFilter(fp)).ToArray()) +
                (addAllFiles ? "|All Files|*.*" : "");
        }

        internal string GetFilter(IDocumentFormatProvider formatProvider)
        {
            return
                formatProvider.FilesDescription +
                " (" +
                string.Join(", ", formatProvider.SupportedExtensions.Select(ext => "*" + ext).ToArray()) +
                ")|" +
                string.Join(";", formatProvider.SupportedExtensions.Select(ext => "*" + ext).ToArray());
        }

        internal void Save(RadDocument document, Stream output, string extension)
        {
            IDocumentFormatProvider provider = DocumentFormatProvidersManager.GetProviderByExtension(extension);

            if (provider == null)
            {
                return;
            }

            try
            {
                using (output)
                {
                    provider.Export(document, output);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(LocalizationManager.GetString("Documents_SaveCommand_UnableToSaveFile"));
            }
        }

    }
}
